// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0
package jobqueue

import (
	"github.com/stretchr/testify/assert"
	"gorm.io/driver/sqlite"
	"gorm.io/gorm"
	"testing"
	"time"
)

func TestWriteToDB4(t *testing.T) {

	gormDB, err := gorm.Open(sqlite.Open("file:unique_id?mode=memory&cache=shared"), &gorm.Config{})
	if err != nil {
		t.Fatalf("an error occurred while opening the database: %v", err)
	}
	t.Cleanup(func() {
		sqlDB, err := gormDB.DB()
		if err != nil {
			t.Fatalf("failed to get generic database object from GORM DB: %v", err)
		}
		_ = sqlDB.Close()
	})

	db := gormDB

	manager := NewManager()
	manager.SetDB(db)
	err = manager.AddWorker(NewLocalWorker(1))
	assert.Nil(t, err)

	err = manager.Start()
	assert.Nil(t, err)

	runner := &CounterRunnable{}
	job := NewJob[CounterResult]("job3", runner)

	id := job.GetID()

	job.mu.Lock()
	job.stats = JobStats{
		JobID:        id,
		RunCount:     20,
		SuccessCount: 30,
		ErrorCount:   40,
	}
	job.mu.Unlock()

	db.Logger = db.Logger.LogMode(4)

	scheduler := &InstantScheduler{}
	err = manager.ScheduleJob(job, scheduler)
	assert.Nil(t, err)

	time.Sleep(100 * time.Millisecond)

	Info("WaitSync")
	manager.WaitSync()
	assert.Equal(t, 21, job.GetStats().RunCount)

	// check the stats are the values above
	var tmpJob JobPersistence

	err = db.Preload("Stats").First(&tmpJob, "id = ?", "job3").Error
	assert.Nil(t, err)

	// Validate the fields

	stats := tmpJob.GetStats()

	assert.Equal(t, JobID("job3"), tmpJob.ID)
	assert.Equal(t, 21, stats.RunCount)
	assert.Equal(t, 31, stats.SuccessCount)
	assert.Equal(t, 40, stats.ErrorCount)

	// reset stats
	err = manager.ResetJobStats(job.GetID())
	assert.Nil(t, err)
	time.Sleep(100 * time.Millisecond)

	var tmpJob2 JobPersistence
	// check is stats are the values above
	err = db.First(&tmpJob2, "id = ?", "job3").Error
	err = db.Preload("Stats").First(&tmpJob2, "id = ?", "job3").Error
	assert.Nil(t, err)

	// Validate the fields

	stats2 := tmpJob2.GetStats()

	assert.Equal(t, JobID("job3"), tmpJob2.ID)
	assert.Equal(t, 0, stats2.RunCount)
	assert.Equal(t, 0, stats2.SuccessCount)
	assert.Equal(t, 0, stats2.ErrorCount)

	err = manager.DeleteJob(job.GetID())
	assert.Nil(t, err)

}
